unit udataeditor;

{$mode objfpc}{$H+}

interface

uses
  sysutils, fileutil, forms, controls, dialogs, ExtCtrls,
  Buttons, StdCtrls, TFileArrayUnit,umainform, Classes;

type

  { Tdataeditorform }

  Tdataeditorform = class(tform)
    cmd_commit: TBitBtn;
    cmd_delete: TBitBtn;
    cmd_change: TBitBtn;
    cmd_close: TBitBtn;
    img_icon: timage;
    lbl_dataedit: TLabel;
    lst_data: TListBox;
    panel1: tpanel;
    procedure cmd_changeclick(sender: tobject);
    procedure cmd_closeClick( Sender: TObject);
    procedure cmd_commitclick(sender: tobject);
    procedure cmd_deleteclick(sender: tobject);
    procedure formclose(sender: tobject; var closeaction: tcloseaction);
    procedure formcreate(sender: tobject);
    procedure formshow(sender: tobject);
    procedure lst_dataclick(sender: tobject);
  private
    { private declarations }
    RecArray:Array of TRecGuess;
    filename:String;
    NumRecs:Integer;
    CurrentSelection:Integer;
    RecordsChanged:Boolean;
    Function ReadDataFile:boolean;
    Function WriteDataFile:Boolean;
    Procedure SplitElement(CONST sElement:String; VAR sAnswer,sQuestion:String);
    Procedure FillListFromRecArray;
  public
    { public declarations }
   end;

var
  dataeditorform: Tdataeditorform;

implementation

{$R *.lfm}

{ Tdataeditorform }
Procedure Tdataeditorform.FillListFromRecArray;
Var
  iCount:Integer;
  sQuestion,sAnswer,sElement:String;
Begin
   lst_data.Items.Clear;
   For iCount:=0 to High(RecArray)-1 do
       begin
            sElement:=RecArray[iCount].Element;
            SplitElement(sElement,sAnswer,sQuestion);
            lst_data.Items.Add(Format('(%d,%d): %s : %s',[RecArray[iCount].Row,RecArray[iCount].Col,sAnswer,sQuestion]));
       end;
end;

Procedure Tdataeditorform.SplitElement(CONST sElement:String; VAR sAnswer,sQuestion:String);
Var
  iSplit:Integer;
Begin
// Raw entry is (SUREMARKER)ANSWER+QUESTION
 iSplit:=Pos(C_ANSWERQUESTIONSPLITTER,sElement);
 If iSplit > 0 then
    begin
         sAnswer:=LeftStr(sElement,iSplit-1);
         sQuestion:=RightStr(sElement,Length(sElement)-iSplit);
    end
    else
    begin
         sAnswer:=sElement;
         sQuestion:='';
    end;
 // Strip Sure Marker if present
 If (Length(sAnswer) > 1) then
       If sAnswer[1] = FileArray.SureMarker then
       sAnswer:=Copy(sAnswer,2,Length(sAnswer)-1);

end;
Function Tdataeditorform.WriteDataFile:boolean;
Var
    F:tFileRecGuess;
    iCount:Integer;
begin
Result:=FALSE;
filename:=FileArray.DataFilePath;
AssignFile(F,filename);
TRY
// Make backup first!
   If FileExists(filename) then
      CopyFile(PChar(filename),PChar(ChangeFileExt(filename,'.bak')),FALSE);
   Rewrite(F); // Always erase existing file
   FileMode := 1;  {Set file access to write only }
   NumRecs:=0;
   for iCount:=0 to High(RecArray) do
       begin
            if RecArray[iCount].Row > 0 then
            begin
               Write(F,RecArray[iCount]);// Write to file
               Inc(NumRecs);
            end;
       end;
FINALLY
   CloseFile(F);
END;
if NumRecs > 0 then
 begin
  Result:=True;
 end;
// ShowMessage(Format('%d records written',[NumRecs]));

end;

Function Tdataeditorform.ReadDataFile:boolean;
Var
    rRecGuess:TRecGuess;
    F:tFileRecGuess;
begin
Result:=FALSE;
filename:=FileArray.DataFilePath;
If NOT FileExists(filename) then Exit;
AssignFile(F,filename);
Numrecs:=0;
FileMode := 0;  {Set file access to read only }
Reset(F);

TRY
  WHILE NOT EOF(f) do
    begin
    // Read into temp record
     Read(F,rRecGuess);

     // Push contents of temp record into Recarray
      SetLength(RecArray,NumRecs+1);
      RecArray[NumRecs].Row:=rRecGuess.Row;
      RecArray[NumRecs].Col:=rRecGuess.Col;
      RecArray[NumRecs].Element:=rRecGuess.Element;
     // Update NumRecs property
      Inc(NumRecs);
    end;

    if NumRecs > 0 then
     begin
      FillListFromRecArray;
      Result:=True;
     end;
    // ShowMessage(Format('%d records read',[NumRecs]));
FINALLY
CloseFile(F);
END;end;

procedure Tdataeditorform.formcreate(sender: tobject);
begin
  If Length(Application.Title) > 0 then
     Caption:=Application.Title;
  If Application.Icon <> Nil then
     Begin
          Icon:=Application.Icon;
          img_Icon.Picture.Icon:=Application.Icon;
     end;
  lbl_dataedit.Caption:='This is a listing directly from ' + filearray.datafilepath + CR;
  lbl_dataedit.Caption:=lbl_dataedit.Caption + 'Format is (Row,Column) : Answer : Question';
end;

procedure tdataeditorform.formshow(sender: tobject);
begin
  If NOT ReadDataFile then
  begin
   ShowMessage('Unable to read data file.  Quitting');
   Close;
  end;
  cmd_commit.Enabled:=False;
  lst_data.selected[0]:=True;
  CurrentSelection:=0;
  RecordsChanged:=False;
end;

procedure tdataeditorform.lst_dataclick(sender: tobject);
Var
  f:Integer;
begin
  CurrentSelection:=0;
  for f:=0 to lst_data.Items.Count do
      If lst_data.Selected[f]=TRUE then
      begin
        CurrentSelection:=f;
        BREAK;
      end;
//      ShowMessage(Format('CurrentSelection=%d',[CurrentSelection]));

end;

procedure tdataeditorform.cmd_commitclick(sender: tobject);
begin
  If MessageDlg('This will change the data file permanently. Are you sure?',
  mtConfirmation,[MBYES,MBCANCEL],0)=mrYes then
    If WriteDataFile then
       ShowMessage(Format('%d records saved to disk OK',[NumRecs]));
  FileArray.LoadFromFile;
  mainform.ResetQuiz;
  Close;
end;

procedure tdataeditorform.cmd_deleteclick(sender: tobject);
Var
   iCount,iRecsDeleted:Integer;
   sElementToDelete:ShortString;
   sQuestion,sAnswer:String;
begin
     sElementToDelete:=RecArray[CurrentSelection].Element;
     SplitElement(sElementToDelete,sAnswer,sQuestion);
     If MessageDlg(Format('Delete all instances of%s%s=%s?',[CR,sQuestion,sAnswer]),mtConfirmation,[MBYES,MBNO,MBCANCEL],0)=mrYes then
     begin
          iRecsDeleted:=0;
          // Strip Sure Marker
          If (Length(sElementToDelete) > 1) then
             If sElementToDelete[1] = FileArray.SureMarker then
                sElementToDelete:=Copy(sElementToDelete,2,Length(sElementToDelete)-1);

          // Zero out all occurences of the selected record
          For iCount:=0 to High(RecArray)-1 do
              begin
                   If (RecArray[iCount].Element=sElementToDelete)
                   OR (RecArray[iCount].Element=(FileArray.SureMarker+sElementToDelete)) then
                   begin
                        RecArray[iCount].Row:=0;
                        RecArray[iCount].Col:=0;
                        RecArray[iCount].Element:='DELETED';
                        Inc(iRecsDeleted);
                   end;
              end;
          FillListFromRecArray;
          ShowMessage(Format('Deleted %d occurrence(s) of%s%s=%s',[iRecsDeleted,CR,sQuestion,sAnswer]));
          cmd_commit.Enabled:=True;
          RecordsChanged:=TRUE;
     end;// ..of confirm yes
end;

procedure tdataeditorform.cmd_changeclick(sender: tobject);
Var
   iCount,iRecsChanged:Integer;
   sElementToChange,sNewElement:ShortString;
   sQuestion,sAnswer,sNewQuestion,sNewAnswer:String;
begin
     sQuestion:='';
     sAnswer:='';
     // Fetch the selected element
     sElementToChange:=RecArray[CurrentSelection].Element;
     // Fetch the Answer and Question part
     SplitElement(sElementToChange,sAnswer,sQuestion);
     // Strip Sure Marker
     If (Length(sElementToChange) > 1) then
       If sElementToChange[1] = FileArray.SureMarker then
       sElementToChange:=Copy(sElementToChange,2,Length(sElementToChange)-1);

     sNewQuestion:=sQuestion;
     sNewAnswer:=sAnswer;
     InputQuery('Make any changes and click OK','Change Answer',sNewAnswer);
     If sAnswer=sNewAnswer then
        messageDlg('Answer unchanged.',mtInformation,[MBOK],0)
     else
         messageDlg('Answer changed OK.',mtInformation,[MBOK],0);

     InputQuery('Make any changes and click OK','Change Question',sNewQuestion);
     If sQuestion=sNewQuestion then
        messageDlg('Question unchanged.',mtInformation,[MBOK],0)
     else
        messageDlg('Question changed OK.',mtInformation,[MBOK],0);

     // Make up the new element
     sNewElement:=sNewAnswer + C_ANSWERQUESTIONSPLITTER + sNewQuestion;

     iRecsChanged:=0;
     // Change all occurences of the selected record, preserving
     // the ones with suremarker
     For iCount:=0 to High(RecArray)-1 do
       begin
         If (RecArray[iCount].Element=sElementToChange) then
         begin
              RecArray[iCount].Element:=sNewElement;
              Inc(iRecsChanged);
         end;
         If (RecArray[iCount].Element=(FileArray.SureMarker + sElementToChange)) then
            begin
               RecArray[iCount].Element:=FileArray.SureMarker+ sNewElement;
               Inc(iRecsChanged);
            end;
       end;
       FillListFromRecArray;
       ShowMessage(Format('Changed %d occurrences of "%s"',[iRecsChanged,sElementToChange]));
       cmd_commit.Enabled:=True;
       RecordsChanged:=TRUE;
end;

procedure Tdataeditorform. cmd_closeClick( Sender: TObject);
begin
     If RecordsChanged=TRUE then
       If MessageDlg('Changes not committed.  Lose changes and close?',
       mtConfirmation,[MBYES,MBCANCEL],0) =mrYes then Close
       else ShowMessage('To save your changes, click the ' + cmd_Commit.Caption + ' button')
     else Close;

end;

procedure tdataeditorform.formclose(sender: tobject;
  var closeaction: tcloseaction);
begin
     mainform.show;  ;
end;

end.

