unit umainform;
(*
  This unit contains the main application functionality
  Code can be freely copied and used but not sold
  Original Author: minesadorada
*)

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus,
  ExtCtrls, Buttons, StdCtrls, ComCtrls, TBitClassUnit, TFileArrayUnit,
  Inifiles,strutils;
CONST C_YES = 0;
CONST C_MAYBEYES = 1;
CONST C_DONTKNOW = 2;
CONST C_MAYBENO = 3;
CONST C_NO = 4;
CONST cr = LineEnding;//CHR(10) + CHR(13);
CONST C_ANSWERQUESTIONSPLITTER = '+'; // Split between answer and qualifying question
CONST C_MAXBITSXXQ = 22; // Linux/Lazarus cannot handle 31 bits, so set it lower.
CONST C_MAXQUESTIONS = 20;
CONST C_DATFILENAME='xxqdata';

type

  { Tmainform }

  Tmainform = class(TForm)
    cmd_reset: TBitBtn;
    cmd_close: TBitBtn;
    cmd_next: TBitBtn;
    grp_spdcontainer: TGroupBox;
    img_icon: TImage;
    lbl_debug: TLabel;
    lbl_question: TLabel;
    MainMenu1: TMainMenu;
    EditMenu: TMenuItem;
    mnu_fileeditdata: TMenuItem;
    mnu_help_sep2: TMenuItem;
    mnu_savetofile: TMenuItem;
    mnu_help_datafiles: TMenuItem;
    mnu_help_programfeatures: TMenuItem;
    mnu_help_sep1: TMenuItem;
    mnu_help_sep0: TMenuItem;
    mnu_help_documentation: TMenuItem;
    mnu_file_deleteitem: TMenuItem;
    seperator: TMenuItem;
    mnu_file_deletedatafile: TMenuItem;
    mnu_file_savedatafile: TMenuItem;
    mnu_categories: TMenuItem;
    mnu_help_gettingstarted: TMenuItem;
    mnu_help: TMenuItem;
    mnu_help_about: TMenuItem;
    mnu_file_reloaddatafile: TMenuItem;
    mnu_file_exit: TMenuItem;
    mnu_file: TMenuItem;
    Panel1: TPanel;
    spd_maybeno: TSpeedButton;
    spd_no: TSpeedButton;
    spd_maybeyes: TSpeedButton;
    spd_yes: TSpeedButton;
    spd_dontknow: TSpeedButton;
    infobar: TStatusBar;
    procedure cmd_resetClick( Sender: TObject);
    procedure cmd_closeClick( Sender: TObject);
    procedure cmd_nextClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mnu_fileeditdataclick(sender: tobject);
    procedure mnu_file_deletedatafileClick(Sender: TObject);
    procedure mnu_file_deleteitemClick( Sender: TObject);
    procedure mnu_file_exitClick(Sender: TObject);
    procedure mnu_file_reloaddatafileClick(Sender: TObject);
    procedure mnu_file_savedatafileClick(Sender: TObject);
    procedure mnu_help_aboutClick(Sender: TObject);
    procedure mnu_help_datafilesClick( Sender: TObject);
    procedure mnu_help_gettingstartedClick(Sender: TObject);
    procedure mnu_help_programfeaturesClick( Sender: TObject);
    procedure mnu_savetofileClick( Sender: TObject);
    procedure spd_dontknowClick(Sender: TObject);
    procedure spd_maybenoClick(Sender: TObject);
    procedure spd_maybeyesClick(Sender: TObject);
    procedure spd_noClick(Sender: TObject);
    procedure spd_yesClick(Sender: TObject);
  private
    { private declarations }
    Vowels:Set of 'a'..'z';
    questions:TSTRINGS;
    questionnumber:Integer;
    QuestionsINI:TIniFile;
    CategoryList:TStrings;
    NewQuizFlag:Boolean;
    NumCategories:Integer;
    CurrentCategoryNumQuestions: Integer;
    CurrentCategory:String;
    iCurrentCategory:Integer;
    compulsorysection:String; // Category that is used to test for valid
    questiondataprefix:string; // questions ini file.  Set in xxq.ini
    ConfigFilePath:String; // Depends on system
    QuestionsIniPath:String; // depends on system
    DocPath:String; // Depends on system
    Procedure FormInit;
    Procedure ReadQuestionsFromIni;
    Procedure ReadCategoriesFromIni;
    Procedure GetCurrentCategoryNumQuestions;
    Procedure initquiz;
    procedure CategoryClick(Sender: TObject);
    procedure EditSubmenuClick(Sender: TObject);
    Procedure MakeGuess;
    Procedure AddNewOne(s:String);
    Procedure CreateINI; // Makes a default questions ini file if none found
    function GetOsVer: string;
  public
    { public declarations }
    helptext:string; // This is shared with another form - hence public
    Procedure ResetQuiz;
  end;

var
  mainform: Tmainform;
  T20:TBitClass; // Object
  FileArray:TFileArray; // Object
  xxqfilepath:String;// Path to data files (from xxq.ini) set in forminit.
  // Note FileArray.DataFilePath can always locate XXQDATA.DAT


implementation

{$R *.lfm}
Uses uhelp,udataeditor;

procedure Tmainform.EditSubMenuClick(Sender: TObject);
Var cmenu:TMenuItem;
begin
  cmenu:=Sender AS tmenuitem;
  {$IFDEF WINDOWS}
  Case cmenu.Tag of
       1:ExecuteProcess('notepad.exe',ConfigFilePath,[]);
       2:ExecuteProcess('notepad.exe',QuestionsIniPath,[]);
       end;
  {$ENDIF}
  {$IFDEF LINUX}
  Case cmenu.Tag of
       1:ExecuteProcess('/usr/local/bin/defaulttexteditor',ConfigFilePath,[]);
       2:ExecuteProcess('/usr/local/bin/defaulttexteditor',QuestionsIniPath,[]);
       end;
  {$ENDIF}
end;



function Tmainform.GetOsVer: string;
begin
 Result:='Unknown OS';
{$IFDEF WIN32}
 Result:='Windows 32bit';
{$ENDIF}
{$IFDEF WIN64}
 Result:='Windows 64bit';
{$ENDIF}
{$IFDEF UNIX}
 Result:='Linux';
{$ENDIF}
end;

Procedure Tmainform.CreateINI;
begin
  QuestionsINI.WriteString(compulsorysection,'0','Is there any pain?');
  QuestionsINI.WriteString(compulsorysection,'1','Can this disease kill you?');
  QuestionsINI.WriteString(compulsorysection,'2','Is it infectious?');
  QuestionsINI.WriteString(compulsorysection,'3','Would a psychiatrist be useful');
  QuestionsINI.WriteString(compulsorysection,'4','Can you be cured?');
  QuestionsINI.WriteString(compulsorysection,'5','Is there any bleeding?');
  QuestionsINI.WriteString(compulsorysection,'6','Should you attend hospital immediately?');
  QuestionsINI.WriteString(compulsorysection,'7','Can you move around freely?');
  FileArray.DeleteDataFile;
  FileArray.SaveToFile; // Make a new blank one

end;

Procedure Tmainform.AddNewOne(s:String);
// Adds a new string to the TFileArray object
// Based on the properties of TBitString object
// Only called if all guesses have failed
Var
   fCount:Integer;
   sAnswer:string;
   sQuestion:String;
begin
     sAnswer:=s;
     sQuestion:='';
     T20.GetUnsureNumberArray; // Get all the numbers appropriate to this set of answers
     // Element ZERO is the sure one, and is always equal to T20.Number
     // all others (if present) are equally unsure

     // Keep asking until user inputs a question
     WHILE (sQuestion = '') DO
     begin
           InputQuery('Help','Please give me a question that would distinguish ' + sAnswer + ' from anything else',squestion);
     end;
     If sQuestion <> '' then s:=sAnswer + C_ANSWERQUESTIONSPLITTER + sQuestion;
     for fCount:=Low(T20.UnsureNumberArray) to High(T20.UnsureNumberArray) do
     begin
          // Pump all the entries into the array one by one
          // Usung the numbers from the UnSureArray
          FileArray.Row:=T20.UnsureNumberArray[fCount];
          FileArray.Col:=FileArray.MaxCol+1; //(MaxCol is always High(Guess[row]);
          // The first number found is always Sure
          FileArray.Sure:=(fCount=0);
          // Set new dynamic Guess[Row,Col]
          FileArray.SetElement(s);
     end; // ..of for
     ShowMessage('Thank you!  I have added "' + sAnswer + '" to the database under category "' + CurrentCategory + '".');
end;

Procedure Tmainform.MakeGuess;
Var
 aList:TSTRINGS;
 f:Integer;
 a,s:String;
 GOTIT:boolean;
 sAnswer:string;
 sQuestion:String;
 iSplit:Integer;
begin
  GOTIT:=FALSE;
  Vowels:=['a','e','i','o','u'];
  s:='';
  // This is the key line in the program!
  FileArray.Row:=T20.Number;
  // It links the binary yes/no responses as a number to the
  // index of the 2-dimensional array holding the guesses

  aList:=TStringList.Create;
  TRY
   // Grab all the guesses into a stringlist (including unsure ones)
   aList.Assign(FileArray.GetElementListFromNumber);
   //DEBUG ShowMessage('Format(Number of Guesses : %d',[aList.Count]));
   If aList.Count > 0  then
     begin
      for f:=0 to aList.Count-1 do // Loop through all the guesses
        begin
         If GOTIT=TRUE then BREAK; // Beeak out of the FOR loop early
         s:=aList[f]; // Grab entry
         // Raw entry is ANSWER+QUESTION
         iSplit:=Pos(C_ANSWERQUESTIONSPLITTER,s);
         If iSplit > 0 then
           begin
                sAnswer:=LeftStr(s,iSplit-1);
                sQuestion:=RightStr(s,Length(s)-iSplit);
                //DEBUG ShowMessage(Format('Question=%s, Answer=%s',[sQuestion,sAnswer]));
           end
           else
           begin
                sAnswer:=s;
                sQuestion:='';
                //DEBUG ShowMessage(Format('Question=%s, Answer=%s',[sQuestion,sAnswer]));
           end;
         //
         // The perfect guess has a 'suremarker'character in front of it.  Remove it now.
         sAnswer:=FileArray.StripSureMarker(sAnswer);
         if LowerCase(sAnswer)[1] IN Vowels then a:='an' else a:='a'; // Make more readable
         //
         // Fetch the supplementary question  here and ask it
         // If already C_MAXQUESTIONS-1 asked, then I LOSE!
         Inc(questionnumber);
         If questionnumber > C_MAXQUESTIONS-1 then
           BEGIN
             ShowMessage(Format('I have asked my %d questions%sI admit I am beaten!',[C_MAXQUESTIONS,CR]));
             BREAK;
           END;

         If MessageDlg(Format('Question %d%s%s?',[questionnumber,CR,sQuestion]),
                     mtConfirmation,
                     [MBYES,MBNO],0)=mrYes then
                     // Now ask the answer
                     Begin
                          Inc(questionnumber);
                          If MessageDlg(Format('Question %d%sIs it %s %s?',[questionnumber,CR,a,sAnswer]),
                          mtConfirmation,
                          [MBYES,MBNO],0)=mrYes then
                          begin
                               GOTIT:=TRUE;// This will break the loop early
                               ShowMessage(Format('I guessed it in only %d questions!',[questionnumber]));
                          end;
                     end;
         // User answered 'no' - loop back for another try
         // until all guesses are done
        end;// ..of for

        // All guesses tried and failed.  Ask the user for the answer
        If GOTIT=FALSE then
           begin
                s:='';
                InputQuery('I am desperate to learn','I have no more guesses! Please type in the answer',s);
                If Length(s) > 0 then AddNewOne(s);
                GOTIT:=TRUE;
           end;
    end
    else
    // No guesses available, or all guesses failed.  Just ask the user for the answer.
    begin
      s:='';
      InputQuery('Beaten','You win! Please type in the answer so I can learn',s);
      If Length(s) > 0 then AddNewOne(s);
    end;
    FINALLY
    aList.Free;
    END; // of TRY
    ResetQuiz;
end;

Procedure Tmainform.GetCurrentCategoryNumQuestions;
Var
   categorykeys:TSTRINGS;
Begin
 // Grab the number of questions in the default category
 categorykeys:=TstringList.Create;
 TRY
    questionsINI.ReadSection(CurrentCategory,categorykeys);
    CurrentCategoryNumQuestions:=categorykeys.Count;
 finally
 categorykeys.Free;
 end;

end;

Procedure Tmainform.ReadCategoriesFromIni;
// Run once from form create
// Grab all the categories and make up the Categories menu entries
Var
  f:Integer;
  cmenu:TMenuItem;
begin
   CategoryList:=TStringList.Create;
 TRY
   TRY
      questionsINI.ReadSections(CategoryList);
      NumCategories:=CategoryList.count;
      for f:=0 to CategoryList.count-1 do
      begin
           cmenu:=TMenuItem.Create(mnu_categories);
           // Only one menu item ckecked at a time
           cmenu.RadioItem:=True;
           cmenu.GroupIndex:=1;
           cmenu.Caption:=CategoryList[f];
           cmenu.Tag:=f;// 0-based number sequence that marks a category
           cmenu.OnClick:=@CategoryClick;// Map all clicks to the same event
           mnu_categories.Add(cmenu);// Add the menu item
           if f=0 then cmenu.checked:=TRUE; // Default
      end;

      iCurrentCategory:=0;// Initialise for first run
      CurrentCategory:=CategoryList[iCurrentCategory];
      GetCurrentCategoryNumQuestions;
   finally
   // No tidying up needed here
   end;

 EXCEPT
  ShowMessage('ReadSectionsFromIni Failed');
  Application.Terminate;
 end;
end;


Procedure Tmainform.CategoryClick(Sender: TObject);
// All the Categories menu entries fire this event
Var
   cmenu:TMenuItem;
//   f:Integer;
//   tempMenu:TMenuItem;
begin
  // Cast the TObject into a menuitem to grab its properties
  cmenu:=Sender AS TmenuItem;
  // Get the Menu Caption
  CurrentCategory:=cmenu.Caption;
  iCurrentCategory:=cmenu.Tag; // Zero-based
  cmenu.checked:=TRUE;
  ResetQuiz;
end;

Procedure Tmainform.ReadQuestionsFromIni;
// Uses the string variable CurrentCategory
Var
  keys:TSTRINGS;
  f:Integer;
begin
TRY
   // Read the INI file and populate the questions stringlist
   keys:=TstringList.Create;
   questions.Clear;
   TRY
     questionsINI.ReadSection(CurrentCategory,keys);
     CurrentCategoryNumQuestions:=keys.Count;

     // We have to be sure the bitarray and filearray can handle the number of questions
     If NumCategories + CurrentCategoryNumQuestions > C_MAXBITSXXQ then
        begin
         ShowMessage('Sorry - too many questions in this Category. Edit en.ini or C_MAXBITSXXQ to correct');
         // Kill the app
         Application.Terminate;;
        end;
     // Assign questions stringlist now
     for f:=0 to keys.Count-1 do
     begin
       questions.Add(questionsINI.ReadString(CurrentCategory,keys[f],'unknown'));
     end;
//DEBUG     showmessage(IntToStr(CurrentCategoryNumQuestions) + ' questions loaded from ' + CurrentCategory);

   FINALLY
   // Free the object.  No longer needed.
          keys.Free;
   END;
EXCEPT
 ShowMessage('ReadQuestionsFromIni Failure');
 Application.Terminate;
END;
end;

procedure Tmainform.FormCreate(Sender: TObject);
begin
  FormInit;
  ReadCategoriesFromIni;
  ResetQuiz;
end;

Procedure Tmainform.ResetQuiz;
// Resets to start-up state
Var
  sStartupMessage:String;
Begin
     T20.ResetAll; // Resets the object to its default (empty) state

     // Set a unique bit to differenciate each category
     T20.Bit:=C_MAXBITSXXQ -iCurrentCategory;
     T20.SwitchBitOn;

     NewQuizFlag:=TRUE;// Used in Next button click

     // Use the main label to display a welcome message
     GetCurrentCategoryNumQuestions;
     sStartupMessage:='Current Category is ' + CurrentCategory + '. (' + IntToStr(CurrentCategoryNumQuestions) + ' questions)';
     sStartupMessage:=sStartupMessage + CR + 'Think of your answer, then click START,';
     sStartupMessage:=sStartupMessage + CR + 'or use the Categories menu to pick a different category.';
     sStartupMessage:=sStartupMessage + CR + CR + 'Be sure to answer the questions as accurately as you can, ';
     sStartupMessage:=sStartupMessage + CR + 'and the program will try to guess your answer in less than 20 questions.';
     sStartupMessage:=sStartupMessage +  ' - Have fun!';
     lbl_question.Caption:=sStartupMessage;

     // Disable all the answer buttons
     spd_yes.Enabled:=False;
     spd_maybeyes.Enabled:=False;
     spd_dontknow.Enabled:=False;
     spd_maybeno.Enabled:=False;
     spd_no.Enabled:=False;

     // Set the main form up ready for a new quiz
     cmd_Next.Caption:='START';
     cmd_reset.Visible:=FALSE;
     InfoBar.SimpleText:= Format('Ready : Total records in database: %d',[FileArray.Recs]);


 lbl_debug.Caption:=IntToStr(T20.Number) + CR + T20.NumberBitString;//T20.NumberBitString
end;

Procedure Tmainform.initquiz;
//Called when START is clicked
begin

  questionnumber:=-1;
  ReadQuestionsFromIni;
  spd_yes.Enabled:=True;
  spd_maybeyes.Enabled:=True;
  spd_dontknow.Enabled:=True;
  spd_maybeno.Enabled:=True;
  spd_no.Enabled:=True;
  cmd_Next.Caption:='NEXT >';
  NewQuizFlag:=FALSE;
  T20.ResetAll;
  // Set a unique bit to differenciate each category
  // The bit should be higher than the highest question bit
  T20.Bit:=C_MAXBITSXXQ -iCurrentCategory;
  T20.SwitchBitOn;

  // Debugging Label invisible
  lbl_debug.Caption:=Format('%d%s%s',[T20.Number,CR,T20.NumberBitString]);
end;

procedure Tmainform.cmd_nextClick(Sender: TObject);
// This routine covers both the START and NEXT clicks
Var
   questiontext,info:String;
begin
  // Debugging Label invisible
  lbl_debug.Caption:=Format('%d%s%s',[T20.Number,CR,T20.NumberBitString]);

  If NewQuizFlag=TRUE then InitQuiz; // START pressed

//Check an answer has been selected only during a quiz
If (questionnumber <> -1)
   AND (spd_yes.Down =False)
   AND (spd_maybeyes.Down =False)
   AND (spd_dontknow.Down =False)
   AND (spd_maybeno.Down =False)
   AND (spd_no.Down =False) then
  begin
    ShowMessage('No answer selected!');
    exit;
  end;

// Next Question
Inc(questionnumber);
If questionNumber >= CurrentCategoryNumQuestions then
  begin
       // All questions have been answered
       cmd_reset.Visible:=FALSE;
       MakeGuess;
       Exit;
  end;

// Display question
questiontext:= questions[questionnumber];
lbl_Question.Caption:=questiontext;
// Populate Status bar

info:=Format('Question %d of %d : Category is %s',[questionnumber+1,CurrentCategoryNumQuestions,CurrentCategory]);
InfoBar.SimpleText:= info;


// Reset the answer buttons
spd_yes.Down:=False;
spd_maybeyes.Down:=False;
spd_dontknow.Down:=False;
spd_maybeno.Down:=False;
spd_no.Down:=False;
cmd_reset.Visible:=TRUE;;
end;

procedure Tmainform.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
   If NOT FileArray.DataSaved then
    If MessageDlg('Save data to disk?',mtConfirmation,[MBYES,MBNO],0)=mrYES then
     If FileArray.SaveToFile then ShowMessage('Data saved OK.')
     else ShowMessage('There was a problem - Data was NOT saved.');

end;

procedure Tmainform. cmd_closeClick( Sender: TObject);
begin
  Close;
end;

procedure Tmainform. cmd_resetClick( Sender: TObject);
begin
  ResetQuiz;
  ShowMessage('Current object (category:' + CurrentCategory + ') abandoned.');
end;

procedure Tmainform.FormDestroy(Sender: TObject);
begin
  // Save any new data and free all objects
  T20.Free;
  FileArray.Free;
  QuestionsINI.Free;
end;

procedure tmainform.mnu_fileeditdataclick(sender: tobject);
begin
  FileArray.SaveToFile;
  Hide;
  dataeditorform.showmodal;
end;

procedure Tmainform.mnu_file_deletedatafileClick(Sender: TObject);
begin
// Destroys data file then builds a new blank one
If MessageDlg('Are you sure?',mtWarning,[MBYES,MBNO],0)=MRYES then
  begin
       if FileArray.DeleteData=TRUE then
          ShowMessage('Data file deleted OK');
       If NOT FileArray.InitOK then
       begin
            // Raise exception and quit if filearray object fails
            raise ETFileArrayException.Create('Could not reinitialise FileArray.  Restart the application.');
            FileArray.Free;
            Application.Terminate;
       end;
       FileArray.SaveToFile;
       MessageDlg('The application needs to be restarted. Click OK, then restart the application',mtInformation,[MBOK],0);
       Close;
; end
  else
      ShowMessage('Operation cancelled OK');

end;

procedure Tmainform. mnu_file_deleteitemClick( Sender: TObject);
(*
  This has been disabled because of a SetLength problem in TFileArray
*)
Var
   s:String;
   i:Integer;
begin
     s:='';
   If InputQuery(Application.Title,'Type in the item you wish to delete',s) then
   begin
        ShowMessage(Format('Before : %d',[FileArray.Recs]));
        i:=FileArray.DeleteAllInstancesOf(s,C_ANSWERQUESTIONSPLITTER);
        ShowMessage(Format('After : %d',[FileArray.Recs]));
    // Using the Format function is much neater!
    ShowMessage(Format('%d instances of %s deleted',[i,s]));
    InfoBar.SimpleText:= Format('Ready : Total records in database: %d',[FileArray.Recs]);
   end
   else
       Showmessage('Delete Cancelled');
end;

procedure Tmainform.mnu_file_exitClick(Sender: TObject);
begin
  Close;
end;

procedure Tmainform.mnu_file_reloaddatafileClick(Sender: TObject);
begin
  FileArray.LoadFromFile;
end;

procedure Tmainform.mnu_file_savedatafileClick(Sender: TObject);
begin
  If FileArray.SaveToFile = TRUE then
    ShowMessage(Format('%d records saved in %s successfully',[FileArray.Recs,FileArray.DataFilePath]))
  else
      Showmessage('Unable to save datafile to disk or datafile is empty');
end;

procedure Tmainform.mnu_help_aboutClick(Sender: TObject);
begin
  // helptext is shared between the forms
  helptext:= 'Written using Lazarus/FPC' + cr + 'by minesadorada Feb 2012' + cr+cr;
  helptext+= 'Lazy Puppy Linux v5.2.8 32-bit' + cr;
  helptext+= 'Microsoft Windows 7 Professional SP1 64-bit' + cr;
  helptext+= 'Microsoft Windows XP Professional SP3 32-bit' + cr + cr;
  helptext+='Config:' + ConfigFilePath + cr +cr;
  helptext+= 'Data: ' + FileArray.DataFilePath + cr;
  helptext+=' and ' + xxqfilepath + questiondataprefix + '.ini'+cr + cr;
  helptext+= 'Docs: ' + docpath;
  hide; // the main form
  helpform.Caption:='About';
  helpform.showmodal; // ShowModal = form must be closed before execution goes
                      // beyond this line.
end;

procedure Tmainform. mnu_help_datafilesClick( Sender: TObject);
begin
 {$IFDEF UNIX}
         ExecuteProcess('/usr/local/bin/defaultwordprocessor',docpath + 'xxq_data_files.rtf',[]);
 {$ENDIF}
 {$IFDEF WINDOWS}
         ExecuteProcess('write.exe',docpath + 'xxq_data_files.rtf',[]);
 {$ENDIF}

end;

procedure Tmainform.mnu_help_gettingstartedClick(Sender: TObject);
begin
     helptext:='This is a 20 questions game that learns as well as guesses.' + CR;
     helptext+='To get started:' + CR;
     helptext+='1) Pick a category from the menu.'+ CR;
     helptext+='2) Think of something in that category.'+ CR;
     helptext+='3) Click Start.'+ CR;
     helptext+='4) Answer each question using the answer buttons'+ CR;
     helptext+='5) If you are not sure, use the most likely button to answer.';
     hide;
     helpform.Caption:='Getting Started';
     helpform.showmodal;

end;

procedure Tmainform. mnu_help_programfeaturesClick( Sender: TObject);
begin
 {$IFDEF UNIX}
         ExecuteProcess('/usr/local/bin/defaultwordprocessor',docpath + 'xxq_features.rtf',[]);
 {$ENDIF}
 {$IFDEF WINDOWS}
         ExecuteProcess('write.exe',docpath + 'xxq_features.rtf',[]);
 {$ENDIF}
end;

procedure Tmainform. mnu_savetofileClick( Sender: TObject);
Var
   AllUniqueElements:TSTRINGS;
   filename:String;
begin
  AllUniqueElements:=TstringList.Create;
  TRY
     AllUniqueElements:=FileArray.GetUniqueElementList;
     filename:=ExtractShortPathNameUTF8(ParamStrUTF8(0)) + 'uniqueelements.txt';
     If FileExistsUTF8(filename) then deletefile(filename);
     Application.Processmessages;
     AllUniqueElements.SaveToFile(filename);
     ShowMessage('Unique elements saved to ' + filename + ' OK.');
  finally
         AllUniqueElements.Free;
  end;

end;

(*
 Answer buttons
 T20.Bit is the actual answer
 T20.UnsureBit is the 'maybe' answer
 The full Bitarray is built up question by question
 The final result is a unique number (T20.Num)
 and a poassible array of 'unsure numbers' (T20.UnsureNumberArray)
*)

procedure Tmainform.spd_dontknowClick(Sender: TObject);
begin
   T20.UnsureBit:=questionnumber;
   T20.SwitchUnsureBitOn;
   T20.Bit:=questionnumber;
   T20.SwitchBitOff;
 end;

procedure Tmainform.spd_maybenoClick(Sender: TObject);
begin
   T20.UnsureBit:=questionnumber;
   T20.SwitchUnsureBitOn;
   T20.Bit:=questionnumber;
   T20.SwitchBitOff;
end;

procedure Tmainform.spd_maybeyesClick(Sender: TObject);
begin
     T20.UnsureBit:=questionnumber;
     T20.SwitchUnsureBitOn;
     T20.Bit:=questionnumber;
     T20.SwitchBitOn;
end;

procedure Tmainform.spd_noClick(Sender: TObject);
begin
  T20.UnsureBit:=questionnumber;
  T20.SwitchUnsureBitOff;
  T20.Bit:=questionnumber;
  T20.SwitchBitOff;
end;



procedure Tmainform.spd_yesClick(Sender: TObject);
begin
     T20.UnsureBit:=questionnumber;
     T20.SwitchUnsureBitOff;
     T20.Bit:=questionnumber;
     T20.SwitchBitOn;
end;

procedure Tmainform.FormInit;
Var
   xxqINI:TiniFile;
   currdir:String;
   EditSubMenu:TMenuItem;
// This is only called once to instantiate all objects etc.
begin
If Application.Icon <> Nil then
 Begin
  Icon:=Application.Icon;
  img_Icon.Picture.Icon:=Application.Icon;
 end;
 T20:=TBitClass.Create;
 If NOT T20.InitOK then
  begin
   raise ETBitClassException.Create(T20.ErrorString);
   T20.Free;
   Application.Terminate;
  end;
  FileArray:=TFileArray.Create(C_MAXBITSXXQ); // Max. No. Of Bits(questions) used here
  If NOT FileArray.InitOK then
  begin
   raise ETFileArrayException.Create('Could not initialise FileArray.  Quitting');
   FileArray.Free;
   Application.Terminate;
  end;
  T20.Number:=0;
  T20.Bit:=0;
  questions:=TStringList.Create;

  // Sort out paths and files. All the code below is to ensure normal
  // operation when first run.

  // BASIC MINIMUM INSTALLATION is the program + xxq.org
  // BASIC INSTALLATION is the above + a pre-populated en.org file
  // NORMAL INSTALLATION is the above + pre-populated XXQDATA.ORG file + rtf files

  // The .ORG files are copied and renamed to appropriate directories
  // depending on the environment.  The .org files are retained, so
  // if the ini and dat files are deleted, or a new user logs on, then
  // the .org files are re-copied and all is well.
  // The rtf files are moved on first run from the current directory

  // Note Current Directory has no meaning in LINUX!
  currdir:=GetCurrentDirUTF8;// alternative: ExtractShortPathNameUTF8(ParamStrUTF8(0));
  currdir:=AppendPathDelim(currdir); // Make ready to append a filename to it
  // Function TrimFileName() sorts out path delimiter errors!
  xxqfilepath:=currdir;// Default

  // Find a suitable filename and path for this platform
  // Note: the config file applies to ALL users (global)
  ConFigExtension:='.ini';
  ConfigFilePath := DelSpace(GetAppConfigFile(true));

  {$IFDEF WINDOWS}
  // The GetAppConfigFile returns a path in APPDATA or other read-only folder
  // so store the config file off the application folder so the user can edit it
          ConfigFilePath :=TrimFileName(currdir + 'config\xxq.ini');
          ForceDirectoriesUTF8(currdir + 'config');
  {$ENDIF}

  If NOT FileExistsUTF8(ConfigFilePath)
  AND FileExistsUTF8(TrimFileName(currdir + 'xxq.org')) then
  CopyFile(TrimFileName(currdir + 'xxq.org'),ConfigFilePath);

  // Read basic configuation information from copy of xxq.org
  TRY
    xxqINI:=TIniFile.Create(ConfigFilePath);
    questiondataprefix:=xxqINI.ReadString('programinfo','questiondataprefix','unknown');
    if questiondataprefix='unknown' then questiondataprefix:='en';// default
    compulsorysection:=xxqINI.ReadString('programinfo','compulsorysection','unknown');
    If compulsorysection='unknown'then compulsorysection:='Diseases'; // default
    {$IFDEF LINUX}
            docpath:=xxqINI.ReadString('programinfo','docpathlinux','unknown');
            xxqfilepath:=xxqINI.ReadString('programinfo','datapathlinux','unknown');
    {$ENDIF}
    {$IFDEF WINDOWS}
            docpath:=xxqINI.ReadString('programinfo','docpathwindows','unknown');
            xxqfilepath:=xxqINI.ReadString('programinfo','datapathwindows','unknown');
    {$ENDIF}
    If xxqfilepath='unknown' then xxqfilepath:=TrimFileName(currdir + PathDelim + 'data');
    If docpath='unknown' then docpath:=TrimFileName(currdir + PathDelim + 'docs');

  EXCEPT
    ShowMessage('Cound not find configuration file ' + ConfigFilePath);
    xxqINI.Free;
    Application.Terminate;
  END;

  xxqfilepath:=AppendPathDelim(xxqfilepath); // fix up the file path
  docpath:=AppendPathDelim(docpath); // fix up the file path

  // On first run, then copy docs and data files from current directory to path specified in XXQ.INI
  If xxqfilepath <> currdir then
  BEGIN
       // The INI file MUST be present and populated.  Use a template .org file if present
       If NOT FileExistsUTF8(TrimFileName(xxqfilepath + questiondataprefix + '.ini')) AND
       FileExistsUTF8(TrimFileName(currdir + questiondataprefix + '.org'))then
       begin
            ForceDirectoriesUTF8(xxqfilepath);
            CopyFile(TrimFileName(currdir + questiondataprefix + '.org'),TrimFileName(xxqfilepath + questiondataprefix + '.ini'));
       end;

       // Move the docs over (move, not copy)
       If NOT FileExistsUTF8(TrimFileName(docpath + 'xxq_data_files.rtf')) AND
       FileExistsUTF8(TrimFileName(currdir + 'xxq_data_files.rtf'))then
       begin
            ForceDirectoriesUTF8(docpath);
            CopyFile(TrimFileName(currdir + 'xxq_data_files.rtf'),TrimFileName(docpath + 'xxq_data_files.rtf'));
            {$IFDEF LINUX}
            DeleteFile(currdir + 'xxq_data_files.rtf');
            {$ENDIF}
       end;
       If NOT FileExistsUTF8(TrimFileName(docpath + 'xxq_features.rtf')) AND
       FileExistsUTF8(TrimFileName(currdir + 'xxq_features.rtf'))then
       begin
            ForceDirectoriesUTF8(docpath);
            CopyFile(TrimFileName(currdir + 'xxq_features.rtf'),TrimFileName(docpath + 'xxq_features.rtf'));
            {$IFDEF LINUX}
            DeleteFile(currdir + 'xxq_features.rtf');
            {$ENDIF}
       end;

       // The DAT file can be empty or absent.
       If NOT FileExistsUTF8(TrimFileName(xxqfilepath + C_DATFILENAME+ '.dat')) AND
       FileExistsUTF8(TrimFileName(currdir + C_DATFILENAME + '.org'))then
       begin
            ForceDirectoriesUTF8(xxqfilepath);
            CopyFile(TrimFileName(currdir + C_DATFILENAME + '.org'),TrimFileName(xxqfilepath + C_DATFILENAME + '.dat'));
       end;

  END;

  // Now sort out the questions ini file
  QuestionsIniPath:=TrimFileName(xxqfilepath + questiondataprefix + '.ini');
  TRY
    QuestionsINI:=TIniFile.Create(QuestionsIniPath);
   EXCEPT
    ShowMessage('Cound not create ' + QuestionsIniPath);
    QuestionsINI.Free;
    Application.Terminate;
   END;
   // Now sort out the DAT file
   // Set the data file location property here
   FileArray.DataFilePath:=TrimFileName(xxqfilepath + C_DATFILENAME + '.dat');
   // Load data.  If no data, then create new data file
   If Not FileArray.LoadFromFile then FileArray.SaveToFile;

   // Test for populated INI file which MUST be present.
   // If it's not there and there was no .org file, then make up a default one.
   // NOTE: Any existing data file is re-created if a new ini file has to be made!
   If QuestionsINI.ReadString(compulsorysection,'0','unknown')='unknown' then CreateINI;

   // Create Edit Menu.  Shelling is unreliable in Linux
   EditMenu.Visible:=True;
   EditSubMenu:=TMenuItem.Create(EditMenu);
   EditSubMenu.Caption:='Edit configuration file (advanced user)';
   EditSubMenu.Tag:=1; // We may want to add more items later
   EditSubmenu.OnClick:=@EditSubMenuClick; // The tag property can be picked up
   EditMenu.Add(EditSubMenu);
   EditSubMenu:=TMenuItem.Create(EditMenu);
   EditSubMenu.Caption:='Edit questions file (advanced user)';
   EditSubMenu.Tag:=2; // We may want to add more items later
   EditSubmenu.OnClick:=@EditSubMenuClick; // The tag property can be picked up
   EditMenu.Add(EditSubMenu);
 If Length(Application.Title) > 0 then
   Application.Title:=Application.Title + ' (' + GetOsVer + ' version)';
	Caption:=Application.Title;
end;
end.

